@@ -3,7 +3,7 @@ use darling::FromMeta;
 use proc_macro2::{Ident, TokenStream};
 use quote::{quote, ToTokens};
 use std::collections::HashMap;
-use syn::{AttributeArgs, Expr, GenericParam, ItemEnum, ItemStruct, ItemType, Lit, Type};
+use syn::{AttributeArgs, Expr, GenericParam, ItemEnum, ItemStruct, ItemType, Lit, Type, Visibility};
 
 #[derive(Debug, FromMeta, Clone)]
 pub struct FFITypeAttributes {
@@ -139,6 +139,7 @@ pub fn ffi_type_struct(attr: FFITypeAttributes, input: TokenStream, item: ItemSt
     let mut field_names = Vec::new();
     let mut field_types = Vec::new();
     let mut field_docs = Vec::new();
+    let mut field_visibility = Vec::new();
     let mut generic_params_needing_bounds = Vec::new();
 
     let mut generics_params = quote! {};
@@ -159,24 +160,27 @@ pub fn ffi_type_struct(attr: FFITypeAttributes, input: TokenStream, item: ItemSt
         generics_params = quote! { < #(#generic_params),* > };
 
         if !generic_params_needing_bounds.is_empty() {
-            if let Some(x) = item.generics.where_clause {
-                // where_clause = quote! { #x, #( #generic_params_needing_bounds: interoptopus::lang::rust::CTypeInfo)*, }
-                where_clause = quote! { #x }
-            } else {
-                // where_clause = quote! { where #( #generic_params_needing_bounds: interoptopus::lang::rust::CTypeInfo)*, }
+            if let Some(where_cl) = item.generics.where_clause {
+                where_clause = quote! { #where_cl }
             }
         }
     }
 
-    for field in &item.fields {
-        let name = field.ident.as_ref().expect("Field must be named").to_string();
+    for (i, field) in item.fields.iter().enumerate() {
+        let name = field.ident.as_ref().map(|x| x.to_string()).unwrap_or_else(|| format!("x{}", i.to_string()));
 
         if attr.skip.contains_key(&name) {
             continue;
         }
 
+        let visibility = match &field.vis {
+            Visibility::Public(_) => quote! { interoptopus::lang::c::Visibility::Public },
+            _ => quote! { interoptopus::lang::c::Visibility::Private },
+        };
+
         field_names.push(name.clone());
         field_docs.push(extract_doc_lines(&field.attrs).join("\n"));
+        field_visibility.push(visibility);
 
         if attr.surrogates.contains_key(&name) {
             let lookup = attr.surrogates.get(&name).unwrap();
@@ -202,18 +206,24 @@ pub fn ffi_type_struct(attr: FFITypeAttributes, input: TokenStream, item: ItemSt
         impl #generics_params interoptopus::lang::rust::CTypeInfo for #name_ident #generics_params #where_clause {
 
             fn type_info() -> interoptopus::lang::c::CType {
-
                 let documentation = interoptopus::lang::c::Documentation::from_line(#doc_line);
                 let mut meta = interoptopus::lang::c::Meta::with_namespace_documentation(#namespace.to_string(), documentation);
                 let mut fields = std::vec::Vec::new();
+                let mut generics: std::vec::Vec<String> = std::vec::Vec::new();
+
+                #({
+                    generics.push(<#generic_params_needing_bounds as interoptopus::lang::rust::CTypeInfo>::type_info().name_within_lib());
+                })*
 
                 #({
                     let documentation = interoptopus::lang::c::Documentation::from_line(#field_docs);
                     let the_type = #field_types;
-                    fields.push(interoptopus::lang::c::Field::with_documentation(#field_names.to_string(), the_type, documentation));
+                    fields.push(interoptopus::lang::c::Field::with_documentation(#field_names.to_string(), the_type, #field_visibility, documentation));
                 })*
 
-                let rval = interoptopus::lang::c::CompositeType::with_meta(#c_struct_name.to_string(), fields, meta);
+                let name = format!("{}{}", #c_struct_name.to_string(), generics.join(""));
+
+                let rval = interoptopus::lang::c::CompositeType::with_meta(name, fields, meta);
                 interoptopus::lang::c::CType::Composite(rval)
             }
         }
